@@ -354,10 +354,7 @@ void async_server_test()
 
 struct error_api
 {
-   std::function<int()> func = []() {
-      throw std::runtime_error("func error");
-      return 0;
-   };
+   std::function<int()> func = []() -> int { throw std::runtime_error("func error"); };
 };
 
 void server_error_test()
@@ -436,14 +433,8 @@ suite send_receive_api_tests = [] {
 
 struct keep_alive_api
 {
-   std::function<int()> broken = []() {
-      throw std::runtime_error("broken");
-      return 0;
-   };
-   std::function<int()> unknown_broken = []() {
-      throw 5;
-      return 0;
-   };
+   std::function<int()> broken = []() -> int { throw std::runtime_error("broken"); };
+   std::function<int()> unknown_broken = []() -> int { throw 5; };
    std::function<int()> works = []() { return 42; };
 };
 
