@@ -1,13 +1,20 @@
 #include "glaze/net/http_client.hpp"
 
+#include <algorithm>
 #include <atomic>
+#include <cctype>
 #include <chrono>
 #include <future>
+#include <optional>
+#include <string_view>
 #include <thread>
 #include <unordered_map>
+#include <utility>
+#include <vector>
 
 #include "glaze/json/write.hpp"
 #include "glaze/net/http_server.hpp"
+#include "glaze/util/key_transformers.hpp"
 #include "ut/ut.hpp"
 
 using namespace ut;
@@ -40,6 +47,8 @@ class working_test_server
 
    ~working_test_server() { stop(); }
 
+   void set_cors_config(glz::cors_config config) { cors_config_ = std::move(config); }
+
    bool start()
    {
       if (running_) return true;
@@ -120,18 +129,27 @@ class working_test_server
    std::thread server_thread_;
    uint16_t port_;
    std::atomic<bool> running_;
+   std::optional<glz::cors_config> cors_config_;
 
    void setup_routes()
    {
       // Set up custom error handler to suppress expected shutdown errors
       server_.on_error([this](std::error_code ec, std::source_location loc) {
          // Only log unexpected errors, not normal shutdown errors
-         if (running_ && ec != asio::error::eof && ec != asio::error::operation_aborted) {
+         if (running_ && ec != make_error_code(asio::error::eof) &&
+             ec != make_error_code(asio::error::operation_aborted)) {
             std::fprintf(stderr, "Server error at %s:%d: %s\n", loc.file_name(), static_cast<int>(loc.line()),
                          ec.message().c_str());
          }
       });
 
+      if (cors_config_) {
+         server_.enable_cors(*cors_config_);
+      }
+      else {
+         server_.enable_cors();
+      }
+
       server_.get("/hello", [](const request&, response& res) {
          res.status(200).content_type("text/plain").body("Hello, World!");
       });
@@ -193,38 +211,63 @@ class working_test_server
       });
 
       // Endpoint that sends data periodically, for testing client disconnect
+      // Uses enable_shared_from_this to avoid circular reference memory leaks
       server_.stream_get("/slow-stream", [](request&, streaming_response& res) {
          res.start_stream(200, {{"Content-Type", "text/plain"}});
-         auto conn = res.stream;
 
-         auto timer = std::make_shared<asio::steady_timer>(conn->socket_->get_executor());
-         auto counter = std::make_shared<int>(0);
+         struct slow_stream_handler : std::enable_shared_from_this<slow_stream_handler>
+         {
+            std::shared_ptr<streaming_connection_interface> conn;
+            std::shared_ptr<asio::steady_timer> timer;
+            int counter = 0;
 
-         // Use shared_ptr to safely handle recursive lambda calls and avoid compiler-specific segfaults
-         auto send_data = std::make_shared<std::function<void(const std::error_code&)>>();
-         *send_data = [conn, timer, counter, send_data](const std::error_code& ec) {
-            if (ec || !conn->is_open() || *counter >= 10) {
-               if (conn->is_open()) conn->close();
-               return;
+            explicit slow_stream_handler(std::shared_ptr<streaming_connection_interface> c)
+               : conn(std::move(c)), timer(std::make_shared<asio::steady_timer>(conn->get_executor()))
+            {}
+
+            void start()
+            {
+               timer->async_wait([self = shared_from_this()](const std::error_code& ec) { self->on_timer(ec); });
             }
 
-            conn->send_chunk("chunk" + std::to_string((*counter)++) + ";",
-                             [conn, timer, send_data](std::error_code write_ec) {
-                                if (write_ec || !conn->is_open()) {
-                                   if (conn->is_open()) conn->close();
-                                   return;
-                                }
-                                timer->expires_after(std::chrono::milliseconds(50));
-                                timer->async_wait(*send_data);
-                             });
+            void on_timer(const std::error_code& ec)
+            {
+               if (ec || !conn->is_open() || counter >= 10) {
+                  if (conn->is_open()) conn->close();
+                  return;
+               }
+
+               conn->send_chunk("chunk" + std::to_string(counter++) + ";",
+                                [self = shared_from_this()](std::error_code write_ec) { self->on_write(write_ec); });
+            }
+
+            void on_write(std::error_code write_ec)
+            {
+               if (write_ec || !conn->is_open()) {
+                  if (conn->is_open()) conn->close();
+                  return;
+               }
+               timer->expires_after(std::chrono::milliseconds(50));
+               timer->async_wait([self = shared_from_this()](const std::error_code& ec) { self->on_timer(ec); });
+            }
          };
-         timer->async_wait(*send_data);
+
+         auto handler = std::make_shared<slow_stream_handler>(res.stream);
+         handler->start();
       });
 
       // Endpoint that immediately returns an error
       server_.stream_get("/stream-error", [](request&, streaming_response& res) {
          res.start_stream(403).close(); // e.g. Forbidden
       });
+
+      // Endpoint that returns HTTP 200 but includes mixed success payloads (Typesense-style)
+      server_.stream_get("/stream-typesense", [](request&, streaming_response& res) {
+         res.start_stream(200, {{"Content-Type", "application/json"}});
+         res.send(R"({"success":false}\n)");
+         res.send(R"({"success":true}\n)");
+         res.close();
+      });
    }
 
    bool is_server_ready()
@@ -235,7 +278,7 @@ class working_test_server
          asio::ip::tcp::socket socket(io);
          asio::ip::tcp::endpoint endpoint(asio::ip::make_address("127.0.0.1"), port_);
 
-         std::error_code ec;
+         asio::error_code ec;
          socket.connect(endpoint, ec);
          if (!ec) {
             socket.close();
@@ -290,17 +333,29 @@ class simple_test_client
       return perform_request("POST", *url_parts, body);
    }
 
+   std::expected<response, std::error_code> options(
+      const std::string& url, const std::vector<std::pair<std::string, std::string>>& extra_headers)
+   {
+      auto url_parts = parse_url(url);
+      if (!url_parts) {
+         return std::unexpected(url_parts.error());
+      }
+
+      return perform_request("OPTIONS", *url_parts, "", extra_headers);
+   }
+
   private:
    asio::io_context io_context_;
    std::thread worker_thread_;
 
-   std::expected<response, std::error_code> perform_request(const std::string& method, const url_parts& url,
-                                                            const std::string& body)
+   std::expected<response, std::error_code> perform_request(
+      const std::string& method, const url_parts& url, const std::string& body,
+      std::vector<std::pair<std::string, std::string>> extra_headers = {})
    {
       std::promise<std::expected<response, std::error_code>> promise;
       auto future = promise.get_future();
 
-      asio::post(io_context_, [this, method, url, body, &promise]() {
+      asio::post(io_context_, [this, method, url, body, headers = std::move(extra_headers), &promise]() mutable {
          try {
             asio::ip::tcp::socket socket(io_context_);
             asio::ip::tcp::resolver resolver(io_context_);
@@ -314,6 +369,10 @@ class simple_test_client
             request += "Host: " + url.host + "\r\n";
             request += "Connection: close\r\n";
 
+            for (const auto& [name, value] : headers) {
+               request += name + ": " + value + "\r\n";
+            }
+
             if (!body.empty()) {
                request += "Content-Length: " + std::to_string(body.size()) + "\r\n";
             }
@@ -342,26 +401,43 @@ class simple_test_client
                return;
             }
 
-            // Skip headers for simplicity
+            response resp;
+
             std::string header_line;
             while (std::getline(response_stream, header_line) && header_line != "\r") {
-               // Skip headers
+               if (!header_line.empty() && header_line.back() == '\r') {
+                  header_line.pop_back();
+               }
+
+               auto colon_pos = header_line.find(':');
+               if (colon_pos == std::string::npos) continue;
+
+               std::string name = header_line.substr(0, colon_pos);
+               std::string value = header_line.substr(colon_pos + 1);
+
+               value.erase(value.begin(), std::find_if(value.begin(), value.end(),
+                                                       [](unsigned char ch) { return !std::isspace(ch); }));
+               value.erase(
+                  std::find_if(value.rbegin(), value.rend(), [](unsigned char ch) { return !std::isspace(ch); }).base(),
+                  value.end());
+
+               auto lower_name = glz::to_lower_case(name);
+               resp.response_headers[lower_name] = value;
             }
 
             // Read body
-            std::error_code ec;
+            asio::error_code ec;
             asio::read(socket, response_buffer, asio::transfer_all(), ec);
 
             std::string response_body{std::istreambuf_iterator<char>(&response_buffer),
                                       std::istreambuf_iterator<char>()};
 
-            response resp;
             resp.status_code = parsed_status->status_code;
             resp.response_body = response_body;
 
             promise.set_value(std::move(resp));
          }
-         catch (const std::exception& e) {
+         catch (const std::exception&) {
             promise.set_value(std::unexpected(std::make_error_code(std::errc::connection_refused)));
          }
       });
@@ -409,6 +485,204 @@ suite working_http_tests = [] {
       std::this_thread::sleep_for(std::chrono::milliseconds(50)); // Clean shutdown
    };
 
+   "cors_preflight_generates_options_response"_test = [] {
+      working_test_server server;
+      expect(server.start()) << "Server should start\n";
+
+      simple_test_client client;
+      std::vector<std::pair<std::string, std::string>> headers = {
+         {"Origin", "http://localhost"},
+         {"Access-Control-Request-Method", "GET"},
+         {"Access-Control-Request-Headers", "X-Test-Header"},
+      };
+
+      auto result = client.options(server.base_url() + "/hello", headers);
+
+      expect(result.has_value()) << "OPTIONS preflight should succeed\n";
+      if (result.has_value()) {
+         expect(result->status_code == 204) << "Preflight should return HTTP 204\n";
+      }
+
+      server.stop();
+      std::this_thread::sleep_for(std::chrono::milliseconds(50)); // Clean shutdown
+   };
+
+   "cors_dynamic_origin_validation"_test = [] {
+      glz::cors_config config;
+      config.allowed_origins.clear();
+      config.allowed_origins_validator = [](std::string_view origin) {
+         return (origin.starts_with("http://") && origin.ends_with(".allowed.local")) ||
+                origin == "http://special.local";
+      };
+
+      working_test_server server;
+      server.set_cors_config(config);
+      expect(server.start()) << "Server should start\n";
+
+      simple_test_client client;
+      std::vector<std::pair<std::string, std::string>> headers = {
+         {"Origin", "http://app.allowed.local"},
+         {"Access-Control-Request-Method", "GET"},
+      };
+      auto allowed = client.options(server.base_url() + "/hello", headers);
+      expect(allowed.has_value()) << "OPTIONS preflight should succeed\n";
+      if (allowed.has_value()) {
+         expect(allowed->status_code == 204) << "Default status should remain 204\n";
+         auto origin_header = allowed->response_headers.find("access-control-allow-origin");
+         expect(origin_header != allowed->response_headers.end()) << "Allow-Origin header should be present\n";
+         if (origin_header != allowed->response_headers.end()) {
+            expect(origin_header->second == "http://app.allowed.local")
+               << "Origin should be echoed for allowed pattern\n";
+         }
+      }
+
+      headers[0].second = "http://special.local";
+      auto allowed_callback = client.options(server.base_url() + "/hello", headers);
+      expect(allowed_callback.has_value()) << "Dynamic callback origin should succeed\n";
+      if (allowed_callback.has_value()) {
+         expect(allowed_callback->status_code == 204);
+      }
+
+      headers[0].second = "http://denied.local";
+      auto denied = client.options(server.base_url() + "/hello", headers);
+      expect(denied.has_value()) << "Request should return a response even when denied\n";
+      if (denied.has_value()) {
+         expect(denied->status_code == 403) << "Denied origin should return 403\n";
+      }
+
+      server.stop();
+      std::this_thread::sleep_for(std::chrono::milliseconds(50));
+   };
+
+   "cors_reflects_headers"_test = [] {
+      glz::cors_config config;
+      config.allowed_origins = {"http://client.local"};
+      config.allowed_methods = {"GET", "HEAD", "POST", "PUT", "DELETE", "PATCH"};
+      config.allowed_headers.clear();
+      config.options_success_status = 200;
+      config.max_age = 123;
+
+      working_test_server server;
+      server.set_cors_config(config);
+      expect(server.start()) << "Server should start\n";
+
+      simple_test_client client;
+      auto result = client.options(server.base_url() + "/hello", {{"Origin", "http://client.local"},
+                                                                  {"Access-Control-Request-Method", "GET"},
+                                                                  {"Access-Control-Request-Headers", "X-Test-Header"}});
+
+      expect(result.has_value()) << "OPTIONS preflight should succeed\n";
+      if (result.has_value()) {
+         expect(result->status_code == 200)
+            << "Configured OPTIONS status should be used (got " << result->status_code << ")\n";
+
+         auto methods_it = result->response_headers.find("access-control-allow-methods");
+         expect(methods_it != result->response_headers.end()) << "Allow-Methods header missing\n";
+         if (methods_it != result->response_headers.end()) {
+            expect(methods_it->second == "GET, HEAD, POST, PUT, DELETE, PATCH");
+         }
+
+         auto headers_it = result->response_headers.find("access-control-allow-headers");
+         expect(headers_it != result->response_headers.end());
+         if (headers_it != result->response_headers.end()) {
+            expect(headers_it->second == "X-Test-Header");
+         }
+
+         auto max_age_it = result->response_headers.find("access-control-max-age");
+         expect(max_age_it != result->response_headers.end());
+         if (max_age_it != result->response_headers.end()) {
+            expect(max_age_it->second == "123");
+         }
+      }
+
+      server.stop();
+      std::this_thread::sleep_for(std::chrono::milliseconds(50));
+   };
+
+   "cors_allow_all_headers_flag"_test = [] {
+      glz::cors_config config;
+      config.allowed_origins = {"http://client.local"};
+      config.allowed_methods = {"*"};
+      config.allowed_headers = {"*"};
+
+      working_test_server server;
+      server.set_cors_config(config);
+      expect(server.start()) << "Server should start\n";
+
+      simple_test_client client;
+      auto result = client.options(server.base_url() + "/hello", {{"Origin", "http://client.local"},
+                                                                  {"Access-Control-Request-Method", "DELETE"},
+                                                                  {"Access-Control-Request-Headers", "X-One"}});
+
+      expect(result.has_value());
+      if (result.has_value()) {
+         auto headers_it = result->response_headers.find("access-control-allow-headers");
+         expect(headers_it != result->response_headers.end());
+         if (headers_it != result->response_headers.end()) {
+            expect(headers_it->second == "*") << "Expected * but got " << headers_it->second << "\n";
+         }
+      }
+
+      server.stop();
+      std::this_thread::sleep_for(std::chrono::milliseconds(50));
+   };
+
+   "cors_preflight_rejects_missing_method"_test = [] {
+      working_test_server server;
+      expect(server.start()) << "Server should start\n";
+
+      simple_test_client client;
+      auto result = client.options(server.base_url() + "/hello",
+                                   {{"Origin", "http://client.local"}, {"Access-Control-Request-Method", "POST"}});
+
+      expect(result.has_value()) << "Preflight request should yield a response\n";
+      if (result.has_value()) {
+         expect(result->status_code == 405)
+            << "Preflight should return 405 when requested method is not implemented (got " << result->status_code
+            << ")\n";
+
+         auto allow_it = result->response_headers.find("allow");
+         expect(allow_it != result->response_headers.end()) << "Allow header must be present\n";
+         if (allow_it != result->response_headers.end()) {
+            expect(allow_it->second.find("GET") != std::string::npos)
+               << "Allow header should list the implemented method\n";
+         }
+      }
+
+      server.stop();
+      std::this_thread::sleep_for(std::chrono::milliseconds(50));
+   };
+
+   "cors_wildcard_with_credentials_echoes_origin"_test = [] {
+      glz::cors_config config;
+      config.allowed_origins = {"*"};
+      config.allow_credentials = true;
+      config.allowed_methods = {"GET", "HEAD", "POST", "PUT", "DELETE", "PATCH"};
+
+      working_test_server server;
+      server.set_cors_config(config);
+      expect(server.start()) << "Server should start\n";
+
+      simple_test_client client;
+      auto result = client.options(server.base_url() + "/hello",
+                                   {{"Origin", "http://auth.local"}, {"Access-Control-Request-Method", "POST"}});
+
+      expect(result.has_value());
+      if (result.has_value()) {
+         auto origin_it = result->response_headers.find("access-control-allow-origin");
+         expect(origin_it != result->response_headers.end());
+         if (origin_it != result->response_headers.end()) {
+            expect(origin_it->second == "http://auth.local");
+         }
+
+         auto credentials_it = result->response_headers.find("access-control-allow-credentials");
+         expect(credentials_it != result->response_headers.end());
+      }
+
+      server.stop();
+      std::this_thread::sleep_for(std::chrono::milliseconds(50));
+   };
+
    "basic_post_request"_test = [] {
       working_test_server server;
       expect(server.start()) << "Server should start\n";
@@ -471,6 +745,15 @@ suite working_http_tests = [] {
       expect(!result.has_value()) << "Connection to closed port should fail\n";
    };
 
+   "http_status_error_category"_test = [] {
+      auto ec = make_http_status_error(502);
+
+      expect(ec.category() == http_status_category());
+      expect(ec.value() == 502);
+      expect(!ec.message().empty());
+      expect(ec.message().find("502") != std::string::npos);
+   };
+
    "concurrent_server_requests"_test = [] {
       working_test_server server;
       expect(server.start()) << "Server should start\n";
@@ -565,7 +848,7 @@ suite glz_http_client_tests = [] {
          received_data.append(data);
       };
       auto on_error = [&](std::error_code ec) {
-         if (ec && ec != asio::error::eof && ec != asio::error::operation_aborted) {
+         if (ec && ec != make_error_code(asio::error::eof) && ec != make_error_code(asio::error::operation_aborted)) {
             error_count++;
          }
       };
@@ -621,7 +904,7 @@ suite glz_http_client_tests = [] {
          data_chunks_received++;
       };
       auto on_error = [&](std::error_code ec) {
-         if (ec && ec != asio::error::eof && ec != asio::error::operation_aborted) {
+         if (ec && ec != make_error_code(asio::error::eof) && ec != make_error_code(asio::error::operation_aborted)) {
             error_count++;
          }
       };
@@ -679,8 +962,14 @@ suite glz_http_client_tests = [] {
 
       auto on_data = [&](std::string_view) { data_received = true; };
       auto on_error = [&](std::error_code ec) {
-         // The client should translate the 4xx/5xx status into this error.
-         expect(ec == std::errc::connection_refused);
+         expect(bool(ec)) << "Expected an error code for HTTP failure";
+
+         auto status = http_status_from(ec);
+         expect(status.has_value()) << "Error should expose HTTP status";
+         if (status) {
+            expect(*status == 403) << "Unexpected HTTP status propagated: " << ec.message();
+         }
+
          error_received = true;
       };
       auto on_connect = [&](const response& headers) {
@@ -709,6 +998,86 @@ suite glz_http_client_tests = [] {
       server.stop();
       std::this_thread::sleep_for(std::chrono::milliseconds(50));
    };
+
+   "streaming_request_with_custom_status_predicate"_test = [] {
+      working_test_server server;
+      expect(server.start()) << "Server should start\n";
+
+      glz::http_client client;
+
+      std::atomic<bool> connected = false;
+      std::atomic<bool> error_received = false;
+      std::promise<void> disconnect_promise;
+      auto disconnect_future = disconnect_promise.get_future();
+
+      auto on_data = [&](std::string_view data) {
+         // Capture any payload for later inspection
+         (void)data;
+      };
+      auto on_error = [&](std::error_code) { error_received = true; };
+      auto on_connect = [&](const response& headers) {
+         expect(headers.status_code == 403);
+         connected = true;
+      };
+      auto on_disconnect = [&]() { disconnect_promise.set_value(); };
+
+      auto conn = client.stream_request({.url = server.base_url() + "/stream-error",
+                                         .on_data = on_data,
+                                         .on_error = on_error,
+                                         .method = "GET",
+                                         .on_connect = on_connect,
+                                         .on_disconnect = on_disconnect,
+                                         .status_is_error = [](int status) { return status >= 500; }});
+      expect(conn != nullptr);
+
+      auto status = disconnect_future.wait_for(std::chrono::seconds(2));
+      expect(status == std::future_status::ready) << "Disconnect was not called\n";
+
+      expect(connected == true) << "on_connect should run";
+      expect(error_received == false) << "Custom predicate should suppress 4xx error";
+
+      server.stop();
+      std::this_thread::sleep_for(std::chrono::milliseconds(50));
+   };
+
+   "streaming_request_custom_predicate_flags_success"_test = [] {
+      working_test_server server;
+      expect(server.start()) << "Server should start\n";
+
+      glz::http_client client;
+
+      std::atomic<bool> error_received = false;
+      std::promise<void> disconnect_promise;
+      auto disconnect_future = disconnect_promise.get_future();
+
+      auto on_data = [&](std::string_view) { /* no-op */ };
+      auto on_error = [&](std::error_code ec) {
+         error_received = true;
+         auto status = http_status_from(ec);
+         expect(status.has_value());
+         if (status) {
+            expect(*status == 200);
+         }
+      };
+      auto on_connect = [&](const response& headers) { expect(headers.status_code == 200); };
+      auto on_disconnect = [&]() { disconnect_promise.set_value(); };
+
+      auto conn = client.stream_request({.url = server.base_url() + "/stream-typesense",
+                                         .on_data = on_data,
+                                         .on_error = on_error,
+                                         .method = "GET",
+                                         .on_connect = on_connect,
+                                         .on_disconnect = on_disconnect,
+                                         .status_is_error = [](int status) { return status == 200; }});
+      expect(conn != nullptr);
+
+      auto status = disconnect_future.wait_for(std::chrono::seconds(2));
+      expect(status == std::future_status::ready);
+      expect(error_received == true);
+
+      server.stop();
+      std::this_thread::sleep_for(std::chrono::milliseconds(50));
+   };
 };
 
 int main()
